@@ -43,12 +43,12 @@
 #include <QDir>
 #include <QFileInfo>
 #include <QFileSystemWatcher>
-#include "qiconloader_p.h"
+#include <QStyle>
+#include <private/xdgiconloader/xdgiconloader_p.h>
 
 LXQtPlatformTheme::LXQtPlatformTheme():
     settingsWatcher_(NULL)
 {
-    // qDebug() << "LXQT Platform Theme loaded";
     // When the plugin is loaded, it seems that the app is not yet running and
     // QThread environment is not completely set up. So creating filesystem watcher
     // does not work since it uses QSocketNotifier internally which can only be
@@ -105,14 +105,12 @@ void LXQtPlatformTheme::loadSettings() {
     if(!fontStr_.isEmpty()) {
         if(font_.fromString(fontStr_))
             QApplication::setFont(font_); // it seems that we need to do this manually.
-        // qDebug() << "font: " << font_.toString();
     }
 
     // FixedFont
     fixedFontStr_ = settings.value(QLatin1String("fixedFont")).toString();
     if(!fixedFontStr_.isEmpty()) {
         fixedFont_.fromString(fixedFontStr_);
-        // qDebug() << "fixedFont: " << fixedFont_.toString();
     }
 
     // mouse
@@ -157,10 +155,14 @@ void LXQtPlatformTheme::onSettingsChanged() {
     loadSettings(); // reload the config file
 
     if(style_ != oldStyle) // the widget style is changed
-        qApp->setStyle(style_); // ask Qt5 to apply the new style
+    {
+        // ask Qt5 to apply the new style
+        if (qobject_cast<QApplication *>(QCoreApplication::instance()))
+            QApplication::setStyle(style_);
+    }
 
     if(iconTheme_ != oldIconTheme) { // the icon theme is changed
-        QIconLoader::instance()->updateSystemTheme(); // this is a private internal API of Qt5.
+        XdgIconLoader::instance()->updateSystemTheme(); // this is a private internal API of Qt5.
     }
 
     // if font is changed
@@ -198,18 +200,20 @@ bool LXQtPlatformTheme::usePlatformNativeDialog(DialogType type) const {
 QPlatformDialogHelper *LXQtPlatformTheme::createPlatformDialogHelper(DialogType type) const {
     return 0;
 }
-
-const QPalette *LXQtPlatformTheme::palette(Palette type) const {
-    return new QPalette();
-}
-
 #endif
 
+const QPalette *LXQtPlatformTheme::palette(Palette type) const {
+
+    if (type == QPlatformTheme::SystemPalette)
+        // the default constructor uses the default palette
+        return new QPalette;
+    return QPlatformTheme::palette(type);
+}
+
 const QFont *LXQtPlatformTheme::font(Font type) const {
-	// qDebug() << "font()" << type << SystemFont;
     if(type == SystemFont && !fontStr_.isEmpty()) {
-		// NOTE: for some reason, this is not called by Qt when program startup.
-		// So we do QApplication::setFont() manually.
+        // NOTE: for some reason, this is not called by Qt when program startup.
+        // So we do QApplication::setFont() manually.
         return &font_;
     }
     else if(type == FixedFont && !fixedFontStr_.isEmpty()) {
@@ -219,7 +223,6 @@ const QFont *LXQtPlatformTheme::font(Font type) const {
 }
 
 QVariant LXQtPlatformTheme::themeHint(ThemeHint hint) const {
-    // qDebug() << "themeHint" << hint;
     switch (hint) {
     case CursorFlashTime:
         return cursorFlashTime_;
@@ -250,14 +253,12 @@ QVariant LXQtPlatformTheme::themeHint(ThemeHint hint) const {
     case ItemViewActivateItemOnSingleClick:
         return QVariant(singleClickActivate_);
     case SystemIconThemeName:
-        qDebug() << "iconTheme" << iconTheme_;
         return iconTheme_;
     case SystemIconFallbackThemeName:
         return "hicolor";
     case IconThemeSearchPaths:
         return xdgIconThemePaths();
     case StyleNames:
-        // qDebug() << "StyleNames";
         return QStringList() << style_;
     case WindowAutoPlacement:
         break;
@@ -283,35 +284,50 @@ QVariant LXQtPlatformTheme::themeHint(ThemeHint hint) const {
         break;
     case ContextMenuOnMouseRelease:
         break;
-#if QT_VERSION >= QT_VERSION_CHECK(5, 3, 0)
-    // this was introduced in Qt 5.3.
     case MousePressAndHoldInterval:
         break;
     case MouseDoubleClickDistance:
         break;
-#endif
     default:
         break;
     }
     return QPlatformTheme::themeHint(hint);
 }
 
+QIconEngine *LXQtPlatformTheme::createIconEngine(const QString &iconName) const
+{
+    return new XdgIconLoaderEngine(iconName);
+}
+ 
 // Helper to return the icon theme paths from XDG.
 QStringList LXQtPlatformTheme::xdgIconThemePaths() const
 {
     QStringList paths;
+    QStringList xdgDirs;
+
     // Add home directory first in search path
     const QFileInfo homeIconDir(QDir::homePath() + QStringLiteral("/.icons"));
     if (homeIconDir.isDir())
         paths.prepend(homeIconDir.absoluteFilePath());
 
-    QString xdgDirString = QFile::decodeName(qgetenv("XDG_DATA_DIRS"));
-    if (xdgDirString.isEmpty())
-        xdgDirString = QLatin1String("/usr/local/share/:/usr/share/");
-    foreach (const QString &xdgDir, xdgDirString.split(QLatin1Char(':'))) {
-        const QFileInfo xdgIconsDir(xdgDir + QStringLiteral("/icons"));
-        if (xdgIconsDir.isDir())
-            paths.append(xdgIconsDir.absoluteFilePath());
+    QString xdgDataHome = QFile::decodeName(qgetenv("XDG_DATA_HOME"));
+    if (xdgDataHome.isEmpty())
+        xdgDataHome = QDir::homePath() + QLatin1String("/.local/share");
+    xdgDirs.append(xdgDataHome);
+
+    QString xdgDataDirs = QFile::decodeName(qgetenv("XDG_DATA_DIRS"));
+    if (xdgDataDirs.isEmpty())
+        xdgDataDirs = QLatin1String("/usr/local/share/:/usr/share/");
+    xdgDirs.append(xdgDataDirs);
+
+    foreach (const QString &s, xdgDirs) {
+        const QStringList r = s.split(QLatin1Char(':'), QString::SkipEmptyParts);
+        foreach (const QString &xdgDir, r) {
+            const QFileInfo xdgIconsDir(xdgDir + QStringLiteral("/icons"));
+            if (xdgIconsDir.isDir())
+                paths.append(xdgIconsDir.absoluteFilePath());
+        }
     }
+    paths.removeDuplicates();
     return paths;
 }
